@@ -3,13 +3,15 @@
 #
 # This module is part of GitPython and is released under
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
-
 import re
-from objects.blob import Blob
-from objects.util import mode_str_to_int
 
 from gitdb.util import hex_to_bin
 
+from .objects.blob import Blob
+from .objects.util import mode_str_to_int
+
+from git.compat import defenc
+
 
 __all__ = ('Diffable', 'DiffIndex', 'Diff')
 
@@ -195,7 +197,7 @@ class Diff(object):
                             """, re.VERBOSE | re.MULTILINE)
     # can be used for comparisons
     NULL_HEX_SHA = "0" * 40
-    NULL_BIN_SHA = "\0" * 20
+    NULL_BIN_SHA = b"\0" * 20
 
     __slots__ = ("a_blob", "b_blob", "a_mode", "b_mode", "new_file", "deleted_file",
                  "rename_from", "rename_to", "diff")
@@ -294,7 +296,7 @@ class Diff(object):
         :param stream: result of 'git diff' as a stream (supporting file protocol)
         :return: git.DiffIndex """
         # for now, we have to bake the stream
-        text = stream.read()
+        text = stream.read().decode(defenc)
         index = DiffIndex()
 
         diff_header = cls.re_header.match
@@ -323,6 +325,7 @@ class Diff(object):
         # :100644 100644 687099101... 37c5e30c8... M    .gitignore
         index = DiffIndex()
         for line in stream:
+            line = line.decode(defenc)
             if not line.startswith(":"):
                 continue
             # END its not a valid diff line
