@@ -4,8 +4,19 @@
 # This module is part of GitPython and is released under
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
 
-from git.test.lib import *
-from git import *
+from git.test.lib import (
+    TestBase,
+    StringProcessAdapter,
+    fixture,
+    assert_equal,
+    assert_true,
+
+)
+
+from git import (
+    Diff,
+    DiffIndex
+)
 
 
 class TestDiff(TestBase):
@@ -54,7 +65,7 @@ class TestDiff(TestBase):
 
         for fixture_name in fixtures:
             diff_proc = StringProcessAdapter(fixture(fixture_name))
-            diffs = Diff._index_from_patch_format(self.rorepo, diff_proc.stdout)
+            Diff._index_from_patch_format(self.rorepo, diff_proc.stdout)
         # END for each fixture
 
     def test_diff_interface(self):
