@@ -15,7 +15,8 @@ import getpass
 
 # NOTE:  Some of the unused imports might be used/imported by others.
 # Handle once test-cases are back up and running.
-from exc import GitCommandError
+from .exc import GitCommandError
+from .compat import MAXSIZE
 
 # Most of these are unused here, but are for use by git-python modules so these
 # don't see gitdb all the time. Flake of course doesn't like it.
@@ -445,7 +446,7 @@ class IndexFileSHA1Writer(object):
 
     def __init__(self, f):
         self.f = f
-        self.sha1 = make_sha("")
+        self.sha1 = make_sha(b"")
 
     def write(self, data):
         self.sha1.update(data)
@@ -489,10 +490,7 @@ class LockFile(object):
     def _has_lock(self):
         """:return: True if we have a lock and if the lockfile still exists
         :raise AssertionError: if our lock-file does not exist"""
-        if not self._owns_lock:
-            return False
-
-        return True
+        return self._owns_lock
 
     def _obtain_lock_or_raise(self):
         """Create a lock file as flag for other instances, mark our instance as lock-holder
@@ -530,7 +528,7 @@ class LockFile(object):
             # on bloody windows, the file needs write permissions to be removable.
             # Why ...
             if os.name == 'nt':
-                os.chmod(lfp, int("0777", 8))
+                os.chmod(lfp, 0o777)
             # END handle win32
             os.remove(lfp)
         except OSError:
@@ -548,7 +546,7 @@ class BlockingLockFile(LockFile):
         can never be obtained."""
     __slots__ = ("_check_interval", "_max_block_time")
 
-    def __init__(self, file_path, check_interval_s=0.3, max_block_time_s=sys.maxint):
+    def __init__(self, file_path, check_interval_s=0.3, max_block_time_s=MAXSIZE):
         """Configure the instance
 
         :parm check_interval_s:
